@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2020 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -206,10 +206,10 @@ make_tmp (mu_stream_t in)
   size_t size = 0, n;
   mu_message_t mesg;
   
-  rc = mu_temp_file_stream_create (&out, NULL, 0);
+  rc = mu_temp_stream_create (&out, 0);
   if (rc)
     {
-      mu_error (_("unable to open temporary file: %s"), mu_strerror (rc));
+      mu_error (_("unable to open temporary stream: %s"), mu_strerror (rc));
       exit (EX_UNAVAILABLE);
     }
 
