@@ -1,5 +1,5 @@
 #  GNU Mailutils -- a suite of utilities for electronic mail
-#  Copyright (C) 2009, 2010, 2011 Free Software Foundation, Inc.
+#  Copyright (C) 2009-2021 Free Software Foundation, Inc.
 #
 #  GNU Mailutils is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -17,21 +17,19 @@
 from mailutils import stream, mailcap
 
 stm = stream.FileStream ("/etc/mailcap")
-stm.open ()
-
 mc = mailcap.Mailcap (stm)
 
 for i, entry in enumerate (mc):
-    print "entry[%d]" % (i + 1)
+    print("entry[%d]" % (i + 1))
 
     # typefield
-    print "\ttypefield: %s" % entry.get_typefield ()
+    print("\ttypefield: %s" % entry.get_typefield ())
 
     # view-command
-    print "\tview-command: %s" % entry.get_viewcommand ()
+    print("\tview-command: %s" % entry.get_viewcommand ())
 
     # fields
     for j, ent in enumerate (entry):
-        print "\tfields[%d]: %s" % ((j + 1), ent)
+        print("\tfields[%d]: %s" % ((j + 1), ent))
 
     print
