@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2003, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "imap4d.h"
 
@@ -49,8 +47,10 @@ imap4d_capability_init ()
   static char *capa[] = {
     "IMAP4rev1",
     "NAMESPACE",
+    "ID",
     "IDLE",
-    "X-VERSION",
+    "LITERAL+",
+    "UNSELECT",
     NULL
   };
   int i;
@@ -62,19 +62,23 @@ imap4d_capability_init ()
 static int
 print_capa (void *item, void *data)
 {
-  util_send (" %s", (char *)item);
+  io_sendf (" %s", (char *)item);
   return 0;
 }
 
 int
-imap4d_capability (struct imap4d_command *command, char *arg MU_ARG_UNUSED)
+imap4d_capability (struct imap4d_session *session,
+	           struct imap4d_command *command, imap4d_tokbuf_t tok)
 {
-  util_send ("* CAPABILITY");
-
-  mu_list_do (capa_list, print_capa, NULL);
+  if (imap4d_tokbuf_argc (tok) != 2)
+    return io_completion_response (command, RESP_BAD, "Invalid arguments");
   
-  imap4d_auth_capability ();
-  util_send ("\r\n");
+  io_sendf ("* CAPABILITY");
 
-  return util_finish (command, RESP_OK, "Completed");
+  mu_list_foreach (capa_list, print_capa, NULL);
+  
+  imap4d_auth_capability (session);
+  io_sendf ("\n");
+
+  return io_completion_response (command, RESP_OK, "Completed");
 }
