@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2019 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -175,7 +175,8 @@ prefix_translate_name (struct namespace_prefix const *pfx, char const *name,
 
       name += pfxlen;
 
-      if (pfx->ns == NS_PERSONAL && mu_c_strcasecmp (name, "INBOX") == 0)
+      if (pfx->ns == NS_PERSONAL && pfxlen == 0 &&
+	  mu_c_strcasecmp (name, "INBOX") == 0)
 	{
 	  tmpl = mu_strdup (auth_data->mailbox);
 	  return tmpl;//FIXME
