@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2011-2019 Free Software Foundation, Inc.
+   Copyright (C) 2011-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -55,7 +55,9 @@ get_msgnum (struct parse_msgnum_env *env, size_t *pn)
   if (msgnum == ULONG_MAX && errno == ERANGE)
     return MU_ERR_PARSE;
   env->s = p;
-  if (env->msgset->mbox && env->maxval && msgnum > env->maxval)
+  if (env->minval && msgnum < env->minval)
+    msgnum = env->minval;
+  if (env->maxval && msgnum > env->maxval)
     msgnum = env->maxval;
   *pn = msgnum;
   return 0;
