@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2011-2019 Free Software Foundation, Inc.
+   Copyright (C) 2011-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -104,10 +104,4 @@ mu_stream_msgset_format (mu_stream_t str, struct mu_msgset_format const *fmt,
     return rc;
   return mu_list_foreach (mset->list, _msgrange_printer, &env);
 }
-
-int
-mu_msgset_print (mu_stream_t str, mu_msgset_t mset)
-{
-  return mu_stream_msgset_format (str, mu_msgset_fmt_imap, mset);
-}
     
