@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009-2020 Free Software Foundation, Inc.
+   Copyright (C) 2009-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
@@ -252,8 +252,7 @@ start_program_filter (int *p, struct _mu_prog_stream *fs, int flags)
 	}
        
       /* Close unneded descripitors */
-      for (i = mu_getmaxfd (); i > 2; i--)
-	close (i);
+      mu_close_fds (3);
 
       /*FIXME: Switch to other uid/gid if desired */
       execvp (fs->progname, fs->argv);
