@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2019 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -313,11 +313,8 @@ mu_str_expand (char **output, char const *input, mu_assoc_t assoc)
     }
   else
     {
-      size_t wordc;
-      char **wordv;
-      
-      mu_wordsplit_get_words (&ws, &wordc, &wordv);
-      *output = wordv[0];
+      *output = ws.ws_wordv[0];
+      ws.ws_wordv[0] = NULL;
     }
   mu_wordsplit_free (&ws);
   return rc;
