@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2005, 2007-2012, 2014-2016 Free Software Foundation,
-   Inc.
+   Copyright (C) 2005-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -442,9 +441,9 @@ _construct_attr_array (size_t *pargc, char ***pargv)
   for (i = 0, mu_iterator_first (itr); !mu_iterator_is_done (itr);
        mu_iterator_next (itr), i++)
     {
-      char **str;
+      char *str;
       mu_iterator_current (itr, (void**) &str); 
-      if ((argv[i] = strdup (*str)) == NULL)
+      if ((argv[i] = strdup (str)) == NULL)
         {
           mu_argcv_free (i, argv);
           return ENOMEM;
@@ -459,29 +458,28 @@ _construct_attr_array (size_t *pargc, char ***pargv)
   return 0;
 }
 
-/* FIXME: Duplicated in other modules */
 static void
 get_quota (mu_off_t *pquota, const char *str)
 {
+  size_t n;
   char *p;
-  mu_off_t quota = strtoul (str, &p, 10);
-  switch (*p)
+  int rc = mu_strtosize (str, &p, &n);
+  switch (rc)
     {
     case 0:
-      break;
-	      
-    case 'k':
-    case 'K':
-      quota *= 1024;
+      *pquota = n;
       break;
       
-    case 'm':
-    case 'M':
-      quota *= 1024*1024;
+    case ERANGE:
+      mu_error (_("quota value is out of allowed range: %s"), str);
       break;
-	      
+
+    case MU_ERR_PARSE:
+      mu_error (_("bad quota value: %s, stopped at %s"), str, p);
+      break;
+
     default:
-      mu_error (_("invalid value for quota: %s"), str);
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_strtosize", str, rc);
     }
 }
 
@@ -518,7 +516,7 @@ _assign_partial_auth_data (struct mu_auth_data *d, const char *key,
     rc = (d->shell = strdup (val)) ? 0 : errno;
   else if (strcmp (key, MU_AUTH_MAILBOX) == 0)
     rc = (d->mailbox = strdup (val)) ? 0 : errno;
-  else if (strcmp (key, MU_AUTH_QUOTA) == 0)   
+  else if (strcmp (key, MU_AUTH_QUOTA) == 0)
     get_quota (&d->quota, val);
   return rc;
 }
@@ -545,17 +543,15 @@ _mu_entry_to_auth_data (LDAP *ld, LDAPMessage *msg,
        mu_iterator_next (itr))
     {
       char *key;
-      char **pattr;
       char *attr;
       struct berval **values;
       
-      mu_iterator_current_kv (itr, (const void **)&key, (void**) &pattr);
-      attr = *pattr;
+      mu_iterator_current_kv (itr, (const void **)&key, (void**) &attr);
       values = ldap_get_values_len (ld, msg, attr);
       if (!values || !values[0])
 	{
 	  mu_error ("LDAP field `%s' (`%s') has NULL value",
-		    key, *pattr);
+		    key, attr);
 	  _free_partial_auth_data (&d);
 	  return MU_ERR_READ;
 	}
@@ -598,38 +594,34 @@ _mu_ldap_search (LDAP *ld, const char *filter_pat, const char *key,
   size_t nattrs;
   LDAPMessage *res, *msg;
   ber_int_t msgid;
-  const char *env[3];
-  struct mu_wordsplit ws;
-
+  char *filter_str;
+  
   rc = _construct_attr_array (&nattrs, &attrs);
   if (rc)
     return rc;
 
-  env[0] = "user";
-  env[1] = key;
-  env[2] = NULL;
-
-  ws.ws_env = env;
-  if (mu_wordsplit (filter_pat, &ws,
-		    MU_WRDSF_NOSPLIT | MU_WRDSF_NOCMD |
-		    MU_WRDSF_ENV | MU_WRDSF_ENV_KV))
+  rc = mu_str_vexpand (&filter_str, filter_pat, "user", key, NULL);
+  if (rc)
     {
-      mu_error (_("cannot expand line `%s': %s"), filter_pat,
-		mu_wordsplit_strerror (&ws));
-      return MU_ERR_FAILURE;
-    }
-  else if (ws.ws_wordc == 0)
-    {
-      mu_error (_("expanding %s yields empty string"), filter_pat);
-      mu_wordsplit_free (&ws);
       mu_argcv_free (nattrs, attrs);
-      return MU_ERR_FAILURE;
+      if (rc == MU_ERR_FAILURE)
+	{
+	  mu_error (_("cannot expand line `%s': %s"), filter_pat,
+		    filter_str);
+	  free (filter_str);
+	}
+      else
+	{
+	  mu_error (_("cannot expand line `%s': %s"), filter_pat,
+		    mu_strerror (rc));
+	}
+      return rc;
     }
   
   rc = ldap_search_ext (ld, ldap_param.base, LDAP_SCOPE_SUBTREE,
-			ws.ws_wordv[0], attrs, 0,
+			filter_str, attrs, 0,
 			NULL, NULL, NULL, -1, &msgid);
-  mu_wordsplit_free (&ws);
+  free (filter_str);
   mu_argcv_free (nattrs, attrs);
 
   if (rc != LDAP_SUCCESS)
@@ -641,7 +633,7 @@ _mu_ldap_search (LDAP *ld, const char *filter_pat, const char *key,
 	return MU_ERR_FAILURE;
     }
 
-  rc = ldap_result (ld, msgid, LDAP_MSG_ALL, NULL, &res );
+  rc = ldap_result (ld, msgid, LDAP_MSG_ALL, NULL, &res);
   if (rc < 0)
     {
       mu_error ("ldap_result failed");
