@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2005, 2007, 2009-2012, 2014-2017 Free Software
-   Foundation, Inc.
+   Copyright (C) 2005-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -45,7 +44,6 @@
 #include <signal.h>
 #include <regex.h>
 #include <mailutils/sieve.h>
-#include <mailutils/mu_auth.h>
 
 /* Build a mime response message from original message MSG. TEXT
    is the message text.
@@ -499,7 +497,7 @@ vacation_subject (mu_sieve_machine_t mach,
 			      _("%lu: cannot compile reply prefix regexp: %s: %s"),
 			      (unsigned long) mu_sieve_get_message_num (mach),
 			      mu_strerror (rc),
-			      err ? err : "");
+			      mu_prstr (err));
 	    }
 	}
 	  
