@@ -1,6 +1,6 @@
 /* Response processing for IMAP client.
    GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2011-2019 Free Software Foundation, Inc.
+   Copyright (C) 2011-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -72,7 +72,7 @@ parse_response_code (mu_imap_t imap, mu_list_t resp)
       if (mu_kwd_xlat_name (mu_imap_response_codes, arg->v.string, &rcode))
 	return -1;
 
-      arg = _mu_imap_list_at (resp, 4);
+      arg = _mu_imap_list_at (resp, 3);
       if (!arg || !_mu_imap_list_element_is_string (arg, "]"))
 	return -1;
     }
@@ -245,8 +245,7 @@ _std_unsolicited_response (mu_imap_t imap, size_t count, mu_list_t resp)
 	  {
 	    if (!rp->action)
 	      mu_debug (MU_DEBCAT_MAILBOX, MU_DEBUG_TRACE9,
-			("%s:%d: ignoring %s response",
-			 __FILE__, __LINE__, rp->name));
+			("ignoring %s response", rp->name));
 	    else
 	      rp->action (imap, resp, NULL);
 	    return 0;
