@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009-2020 Free Software Foundation, Inc.
+   Copyright (C) 2009-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@ print_stream_envelope (mu_stream_t str, msgset_t *mspec, mu_message_t msg,
   if (status)
     {
       mu_error (_("%lu: Cannot get envelope"),
-		(unsigned long) mspec->msg_part[0]);
+		(unsigned long) msgset_msgno (mspec));
     }
   else
     {
