@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2019 Free Software Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -406,8 +406,7 @@ quote0 (msgset_t *mspec, mu_message_t mesg, void *data)
   mu_stream_t flt;
   char *argv[3];
 
-  mu_printf (_("Interpolating: %lu\n"),
-		    (unsigned long) mspec->msg_part[0]);
+  mu_printf (_("Interpolating: %lu\n"), (unsigned long) msgset_msgno (mspec));
 
   mailvar_get (&prefix, mailvar_name_indentprefix, mailvar_type_string, 0);
 
