@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2003-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +12,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Functions shared between comp and forw utilities */
 
@@ -29,54 +27,44 @@ static char *default_format_str =
 "--------\n";
 
 void
-mh_comp_draft (const char *formfile, const char *defformfile,
-	       const char *draftfile)
+mh_comp_draft (const char *formfile, const char *draftfile)
 {
-  char *s = NULL;
-  if (formfile)
+  char *s;
+
+  if (mh_find_file (formfile, &s) == 0)
     {
-      s = mh_expand_name (MHLIBDIR, formfile, 0);
-      if (mh_file_copy (s, draftfile) == 0)
+      if (mh_file_copy (s, draftfile))
 	exit (1);
+      free (s);
     }
   else
     {
-      s = mh_expand_name (MHLIBDIR, defformfile, 0);
-      if (access (s, R_OK) == 0)
+      /* I doubt if we need that: */
+      int rc;
+      mu_stream_t stream;
+      
+      if ((rc = mu_file_stream_create (&stream,
+				       draftfile,
+				       MU_STREAM_WRITE|MU_STREAM_CREAT)))
 	{
-	  if (mh_file_copy (s, draftfile))
-	    exit (1);
+	  mu_error (_("cannot open output file \"%s\": %s"),
+		    draftfile, mu_strerror (rc));
+	  exit (1);
 	}
-      else
+      
+      rc = mu_stream_write (stream, 
+			    default_format_str,
+			    strlen (default_format_str), NULL);
+      mu_stream_close (stream);
+      mu_stream_destroy (&stream);
+      
+      if (rc)
 	{
-	  int rc;
-	  mu_stream_t stream;
-	  
-	  if ((rc = mu_file_stream_create (&stream,
-					draftfile,
-					MU_STREAM_WRITE|MU_STREAM_CREAT)) != 0
-	      || (rc = mu_stream_open (stream)))
-	    {
-	      mu_error (_("cannot open output file \"%s\": %s"),
-			draftfile, mu_strerror (rc));
-	      exit (1);
-	    }
-	  
-	  rc = mu_stream_sequential_write (stream, 
-					default_format_str,
-					strlen (default_format_str));
-	  mu_stream_close (stream);
-	  mu_stream_destroy (&stream, mu_stream_get_owner (stream));
-
-	  if (rc)
-	    {
-	      mu_error (_("error writing to \"%s\": %s"),
-			draftfile, mu_strerror (rc));
-	      exit (1);
-	    }
+	  mu_error (_("error writing to \"%s\": %s"),
+		    draftfile, mu_strerror (rc));
+	  exit (1);
 	}
     }
-  free (s);
 }
 
 int 
@@ -85,6 +73,9 @@ check_draft_disposition (struct mh_whatnow_env *wh, int use_draft)
   struct stat st;
   int disp = DISP_REPLACE;
 
+  if (wh->nowhatnowproc)
+    return disp;
+  
   /* First check if the draft exists */
   if (stat (wh->draftfile, &st) == 0)
     {
@@ -93,7 +84,7 @@ check_draft_disposition (struct mh_whatnow_env *wh, int use_draft)
       else
 	{
 	  printf (ngettext ("Draft \"%s\" exists (%s byte).\n",
-                            "Draft \"%s\" exists (%su bytes).\n",
+                            "Draft \"%s\" exists (%s bytes).\n",
                             (unsigned long) st.st_size),
 		  wh->draftfile, mu_umaxtostr (0, st.st_size));
 	  disp = mh_disposition (wh->draftfile);
