@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009-2012, 2014-2017 Free Software Foundation, Inc.
+   Copyright (C) 2009-2021 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -25,49 +25,18 @@ _repr (PyObject *self)
 {
   char buf[80];
   sprintf (buf, "<" PY_MODULE "." PY_CSNAME " instance at %p>", self);
-  return PyString_FromString (buf);
+  return PyUnicode_FromString (buf);
 }
 
 static PyTypeObject PyFolderType = {
-  PyObject_HEAD_INIT(NULL)
-  0,                         /* ob_size */
-  PY_MODULE "." PY_CSNAME,   /* tp_name */
-  sizeof (PyFolder),         /* tp_basicsize */
-  0,                         /* tp_itemsize */
-  (destructor)_py_dealloc,   /* tp_dealloc */
-  0,                         /* tp_print */
-  0,                         /* tp_getattr; __getattr__ */
-  0,                         /* tp_setattr; __setattr__ */
-  0,                         /* tp_compare; __cmp__ */
-  _repr,                     /* tp_repr; __repr__ */
-  0,                         /* tp_as_number */
-  0,                         /* tp_as_sequence */
-  0,                         /* tp_as_mapping */
-  0,                         /* tp_hash; __hash__ */
-  0,                         /* tp_call; __call__ */
-  _repr,                     /* tp_str; __str__ */
-  0,                         /* tp_getattro */
-  0,                         /* tp_setattro */
-  0,                         /* tp_as_buffer */
-  Py_TPFLAGS_DEFAULT,        /* tp_flags */
-  "",                        /* tp_doc */
-  0,                         /* tp_traverse */
-  0,                         /* tp_clear */
-  0,                         /* tp_richcompare */
-  0,                         /* tp_weaklistoffset */
-  0,                         /* tp_iter */
-  0,                         /* tp_iternext */
-  0,                         /* tp_methods */
-  0,                         /* tp_members */
-  0,                         /* tp_getset */
-  0,                         /* tp_base */
-  0,                         /* tp_dict */
-  0,                         /* tp_descr_get */
-  0,                         /* tp_descr_set */
-  0,                         /* tp_dictoffset */
-  0,                         /* tp_init */
-  0,                         /* tp_alloc */
-  0,                         /* tp_new */
+  .ob_base = { PyObject_HEAD_INIT(NULL) },
+  .tp_name = PY_MODULE "." PY_CSNAME,
+  .tp_basicsize = sizeof (PyFolder),
+  .tp_dealloc = (destructor)_py_dealloc,
+  .tp_repr = _repr,
+  .tp_str = _repr,
+  .tp_flags = Py_TPFLAGS_DEFAULT,
+  .tp_doc = "",
 };
 
 PyFolder *
@@ -87,7 +56,7 @@ api_folder_create (PyObject *self, PyObject *args)
     return NULL;
 
   status = mu_folder_create (&py_folder->folder, name);
-  return _ro (PyInt_FromLong (status));
+  return _ro (PyLong_FromLong (status));
 }
 
 static PyObject *
@@ -112,7 +81,7 @@ api_folder_open (PyObject *self, PyObject *args)
     return NULL;
 
   status = mu_folder_open (py_folder->folder, MU_STREAM_READ);
-  return _ro (PyInt_FromLong (status));
+  return _ro (PyLong_FromLong (status));
 }
 
 static PyObject *
@@ -125,7 +94,7 @@ api_folder_close (PyObject *self, PyObject *args)
     return NULL;
 
   status = mu_folder_close (py_folder->folder);
-  return _ro (PyInt_FromLong (status));
+  return _ro (PyLong_FromLong (status));
 }
 
 static PyObject *
@@ -161,7 +130,7 @@ api_folder_set_authority (PyObject *self, PyObject *args)
     }
 
   status = mu_folder_set_authority (py_folder->folder, py_auth->auth);
-  return _ro (PyInt_FromLong (status));
+  return _ro (PyLong_FromLong (status));
 }
 
 static PyObject *
@@ -192,10 +161,10 @@ folderdata_extractor (void *data, PyObject **dst)
   snprintf (separator, sizeof (separator), "%c", resp->separator);
 
   *dst = PyTuple_New (4);
-  PyTuple_SetItem (*dst, 0, PyString_FromString (attr));
-  PyTuple_SetItem (*dst, 1, PyInt_FromLong (resp->level));
-  PyTuple_SetItem (*dst, 2, PyString_FromString (separator));
-  PyTuple_SetItem (*dst, 3, PyString_FromString (resp->name));
+  PyTuple_SetItem (*dst, 0, PyUnicode_FromString (attr));
+  PyTuple_SetItem (*dst, 1, PyLong_FromLong (resp->depth));
+  PyTuple_SetItem (*dst, 2, PyUnicode_FromString (separator));
+  PyTuple_SetItem (*dst, 3, PyUnicode_FromString (resp->name));
   return 0;
 }
 
@@ -203,21 +172,21 @@ static PyObject *
 api_folder_list (PyObject *self, PyObject *args)
 {
   int status = 0;
-  Py_ssize_t max_level = 0;
+  Py_ssize_t max_depth = 0;
   char *dirname, *pattern;
   PyFolder *py_folder;
   PyObject *py_list;
   mu_list_t c_list = NULL;
 
   if (!PyArg_ParseTuple (args, "O!zs|n", &PyFolderType, &py_folder,
-			 &dirname, &pattern, &max_level))
+			 &dirname, &pattern, &max_depth))
     return NULL;
-  if (max_level < 0)
+  if (max_depth < 0)
     {
       PyErr_SetString (PyExc_RuntimeError, "max level out of range");
       return NULL;
     }
-  status = mu_folder_list (py_folder->folder, dirname, pattern, max_level,
+  status = mu_folder_list (py_folder->folder, dirname, pattern, max_depth,
 			   &c_list);
 
   if (c_list)
@@ -256,6 +225,15 @@ static PyMethodDef methods[] = {
   { NULL, NULL, 0, NULL }
 };
 
+static struct PyModuleDef moduledef = {
+  PyModuleDef_HEAD_INIT,
+  PY_MODULE,
+  NULL,
+  -1,
+  methods
+};
+
+
 int
 mu_py_init_folder (void)
 {
@@ -267,7 +245,7 @@ void
 _mu_py_attach_folder (void)
 {
   PyObject *m;
-  if ((m = _mu_py_attach_module (PY_MODULE, methods)))
+  if ((m = _mu_py_attach_module (&moduledef)))
     {
       Py_INCREF (&PyFolderType);
       PyModule_AddObject (m, PY_CSNAME, (PyObject *)&PyFolderType);
