@@ -1,5 +1,5 @@
 #  GNU Mailutils -- a suite of utilities for electronic mail
-#  Copyright (C) 2009-2012, 2014-2017 Free Software Foundation, Inc.
+#  Copyright (C) 2009-2021 Free Software Foundation, Inc.
 #
 #  This library is free software; you can redistribute it and/or
 #  modify it under the terms of the GNU Lesser General Public
@@ -64,7 +64,7 @@ class Message:
         elif name == 'lines':
             return self.get_lines ()
         else:
-            raise AttributeError, name
+            raise AttributeError (name)
 
     def __len__ (self):
         return self.get_size ()
