@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2003, 2005-2012, 2014-2018 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999-2021 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -303,7 +302,7 @@ static struct mu_cli_setup cli = {
   options,
   sieve_cfg_param,
   N_("GNU sieve -- a mail filtering tool."),
-  "SCRIPT",
+  N_("SCRIPT"),
   NULL,
   N_("Sieve-specific debug levels:\n\
 \n\
